# Copyright (c) 2025 Daniel Paredes (daleonpz)
# SPDX-License-Identifier: Apache-2.0
import json
from rich.table import Table
from rich.console import Console
from pathlib import Path

from typing import List, Optional, Callable, Union
from rich.style import Style

console = Console()

class ReportWriter:
    def __init__(self, report_data ):
        self.report_data = report_data
        self.fields = ["REQ-ID", "Status", "suite", "linked_test"]
        self.console = Console()

    @staticmethod
    # [ [ header1, header2, header3 ], [ row1col1, row1col2, row1col3 ], ... ]
    def print_table_console(data: List[List[str]], 
                            title: str = "Report", 
                            with_title: bool = True, 
                            cell_styles: Optional[dict] = {},
                            row_style_cb: Optional[Callable[[List[str], int], Union[str, Style, None]]] = None
                            ):
        """Prints a table to the console using Rich."""

        table = Table(title=title if with_title else None, show_header=True, header_style="bold magenta")

        if not data:
            print("No data to display.")
            return

        headers = data[0]
        for header in headers:
            table.add_column(header, style="cyan")
        data_rows = data[1:] if len(data) > 1 else []

        # Add rows with styling
        for row_idx, row in enumerate(data_rows):
            # Prepare styled row cells
            styled_row = []
            for cell in row:
                style = cell_styles.get(cell, None)
                styled_cell = f"[{style}]{cell}[/]" if style else cell
                styled_row.append(styled_cell)

            row_style = None
            if row_style_cb:
                row_style = row_style_cb(row, row_idx)

            table.add_row(*styled_row, style=row_style)

        console.print(table)

    @staticmethod
    def print_table_json(data: List):
        """Prints a table to the console in JSON format."""
        console.print_json(data=data, indent=2)

    def write_console(self, domain: str = None, req_type: str = None, detail: bool = False):
        """Pretty-prints the report to the console using Rich."""

        console.print(f"[blink][bold green]Traceability Report - {self.report_data['timestamp']}[/blink][/bold green]", justify="center")

        coverage_table = []
        coverage_table.append([
            "Total Requirements",
            "Tested Requirements",
            "Pass Rate",
            "Coverage",])

        if domain or req_type:
            print(f"Filtering coverage for domain: {domain}, type: {req_type}")
            if domain:
                coverage = self.report_data["detailed_report"]["domains"].get(domain, {})
            elif req_type:
                coverage = self.report_data["detailed_report"]["types"].get(req_type, {})
        else :
            coverage = self.report_data["coverage"]

        coverage_table.append([
            str(coverage.get("total_requirements", 0)),
            str(coverage.get("tested_requirements", 0)),
            f"{coverage.get('pass_rate', 0)}%",
            f"{coverage.get('coverage_rate', 0)}%"
        ])

        self.print_table_console(coverage_table, title="Coverage Summary", with_title=True)

        traceability_report = []
        traceability_report.append(self.fields)

        if domain or req_type:
            filtered_report = {}
            for req_id, data in self.report_data["report"].items():
                if (domain and data.get("domain") == domain) or (req_type and data.get("type") == req_type):
                    filtered_report[req_id] = data
            self.report_data["report"] = filtered_report

        for req_id, data in self.report_data["report"].items():
            # sometimes the values are [] and sometimes they are not
            test_results = data.get("test_results", [])
            for test in test_results:
                suite, test_name, status = test
                traceability_report.append([
                    req_id,
                    status if status else "-",
                    suite if suite else "-",
                    test_name if test_name else "-",
                ])

            ## if test_results is empty, we still want to show the requirement
            if not test_results:
                traceability_report.append([
                    req_id,
                    "NOT TESTED",
                    "-",
                    "-",
                ])

        # Define cell styles
        cell_styles = {
            "FAIL": Style(color="white", blink=True, bold=True),
            "PASS": "green",
            "NOT TESTED": "yellow",
        }

        # Define row style callback (e.g., yellow background for rows where Status is "FAIL")
        def row_style_callback(row: List[str], row_idx: int) -> Optional[str]:
            return "on red" if row[1] == "FAIL" else None

        self.print_table_console(traceability_report, title="Requirement Traceability Report", with_title=True, 
                                 cell_styles=cell_styles, row_style_cb=row_style_callback)

        if detail and not (domain or req_type):
            detailed_stats = self.report_data.get("detailed_report", {})
            domain_tables = [["Domain", "Total Requirements", "Number of Tested Requirements", "Pass Rate", "Coverage Rate"]]
            for domain, val in detailed_stats["domains"].items():
                domain_tables.append([
                    domain,
                    str(val["total_requirements"]),
                    str(val["tested_requirements"]),
                    f"{val['pass_rate']}%",
                    f"{val['coverage_rate']}%"
                ])

                type_tables= [["Type", "Total Requirements", "Number of Tested Requirements", "Pass Rate", "Coverage Rate"]]
                for typ, val in detailed_stats["types"].items():
                    type_tables.append([
                        typ,
                        str(val["total_requirements"]),
                        str(val["tested_requirements"]),
                        f"{val['pass_rate']}%",
                        f"{val['coverage_rate']}%"
                    ])

            self.print_table_console(domain_tables, title="Domain Statistics", with_title=True)
            self.print_table_console(type_tables, title="Type Statistics", with_title=True)

    def write_json(self, output_path: str, 
                   domain: str = None, 
                   req_type: str = None, 
                   detail: bool = False):
        """Writes the report to a JSON file."""
        output_file = Path(output_path)
        output_file.parent.mkdir(parents=True, exist_ok=True)

        with open(output_file, "w") as f:
            json.dump(self.report_data, f, indent=4)

    def write(self, fmt: str,
              output_path: str = "",
              domain: str = None,
              req_type: str = None,
              detail: bool = False):
        """Dispatch writer based on fmt."""
        output_path = output_path + "." + fmt
        if fmt == "console":
            self.write_console(domain=domain, req_type=req_type, detail=detail)
        elif fmt == "json":
            self.write_json(output_path)
            self.console.print(f"[green]Traceability JSON report saved to {output_path}[/green]")
        else:
            self.console.print(f"[red]Unsupported fmt: {fmt}[/red]")




# Copyright (c) 2025 Daniel Paredes (daleonpz)
# SPDX-License-Identifier: Apache-2.0

# TODO: should i update the yaml always after each command, bzw after each update of the database?
#       use sync_to_yaml always after each command?
#       1. rename variables as _XXX to indicate they are private
from raiz.utils.database import RequirementsDB
from raiz.utils.report_generator import ReportWriter

import typer
import os
from pathlib import Path
from robot.api import ExecutionResult
from typing import Optional
import yaml
from collections import defaultdict
from datetime import datetime
import logging
import json
from rich.traceback import install
install(show_locals=True)

db = RequirementsDB()

REQS_DIR = Path("requirements")
REQS_DIR.mkdir(exist_ok=True)
REQ_FILE = REQS_DIR / "requirements.yaml"

logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(levelname)s - %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
)

def add():
    """Add a new requirement interactively."""
    description = typer.prompt("Description")
    req_type = typer.prompt("Type [functional, non-functional, constraint]")
    domain = typer.prompt("Domain [e.g., logging, ble, data-processing]")

    db.add_requirement(description=description, req_type=req_type, domain=domain)

def remove(
    req_id: int = typer.Argument(..., help="Requirement ID to remove (e.g., 1)")
):
    """Remove a requirement by ID and renumber the rest."""
    if req_id < 1:
        typer.echo("Requirement ID must be greater than 0.")
        raise typer.Exit(code=1)

    db.rm(req_id)

def update(
    req_id: int = typer.Argument(..., help="Requirement ID to update (e.g., 1)"),
    field: Optional[str] = typer.Option(None, '--field', '-f', help="Field to update (description, type, domain)"),
    value: Optional[str] = typer.Option(None, '--value', '-v', help="New value for the field")
):
    """Update a requirement by REQ ID. Interactive or field-based."""
    db.update_requirement(req_id, field, value)

def show_requirements(
    req_type: Optional[str] = typer.Option(None,'--type', help="Filter by requirement type (functional, non-functional, constraint)"), 
    domain: Optional[str] = typer.Option(None, help="Filter by requirement domain (e.g., logging, ble, data-processing)"),
    json: Optional[bool] = typer.Option(False, help="Output in JSON format")
):
    """Show requirements with optional filtering by type/domain."""
    db.show_requirements(req_type, domain, json)

def show_types():
    """Show all unique types."""
    db.show_characteristic("type")

def show_domains():
    """Show all unique domains."""
    db.show_characteristic("domain")

def sync_to_yaml( 
        file: Optional[Path] = typer.Option(REQ_FILE, help="Path to output YAML file")
):
    """Dump current SQLite requirements to YAML."""
    reqs = db.get_requirements()

    if file:
        REQ_FILE = Path(file)
        typer.echo(f"Using provided YAML file: {REQ_FILE}")

    if isinstance(reqs, defaultdict):
        reqs = dict(reqs)

    # flat list for YAML it's consistent with the original format
    flat_list = []
    for req_id, req_data in reqs.items():
        entry = {"id": req_id}
        entry.update(req_data)
        flat_list.append(entry)

    with open(REQ_FILE, "w") as f:
        yaml.dump(flat_list, f, default_flow_style=False, sort_keys=False)

    typer.echo(f"Exported {len(reqs)} requirements to {REQ_FILE}")

def sync_from_yaml(
    file: Optional[Path] = typer.Option(REQ_FILE, help="Path to YAML file with requirements")
):
    """Import requirements from YAML into SQLite (overwrites DB)."""
    if file:
        REQ_FILE = Path(file)
        typer.echo(f"Using provided YAML file: {REQ_FILE}")

    if not REQ_FILE.exists():
        typer.echo(f"Error: {REQ_FILE} not found.")
        raise typer.Exit()

    with open(REQ_FILE, "r") as f:
        reqs = yaml.safe_load(f)

    db.create_from_yaml(reqs)
    typer.echo(f"Requirements imported from {REQ_FILE}")


def _default_stat():
    return {
        "total_requirements": 0,
        "tested_requirements": 0,
        "total_tests": 0,
        "passed_tests": 0,
        "pass_rate": 0.0,
        "coverage_rate": 0.0
    }

def _finalize(stat):
    if stat["total_requirements"]:
        stat["coverage_rate"] = round(stat["tested_requirements"] / stat["total_requirements"] * 100, 2)
        if stat["total_tests"]:
            stat["pass_rate"] = round(stat["passed_tests"] / stat["total_tests"] * 100, 2)
            # Remove raw passed_tests and total_tests if not needed
            del stat["passed_tests"]
            del stat["total_tests"]
def trace(
    output: str = typer.Option("traceability", help="Output report filename"),
    robot_output: str = typer.Option("output.xml", help="Path to Robot Framework output.xml"),
    fmt: str = typer.Option("json", help="Output format: console, json"),
    domain: Optional[str] = typer.Option(None, help="Filter by domain (only in console)"),
    req_type: Optional[str] = typer.Option(None,'--type', help="Filter by requirement type (only in console)"),
    detail: bool = typer.Option(False, help="Show detailed trace report (only in console)")
):
    """
    Generate traceability report linking REQ-IDs to Robot tests.
    """
    if not Path(robot_output).exists():
        typer.echo(f"Robot output file '{robot_output}' not found.")
        raise typer.Exit()

    result = ExecutionResult(robot_output)

    req_map = defaultdict(list)

    root_source = result.suite.source

    for suite in result.suite.suites:
        suite_filename = os.path.basename(suite.source or root_source)
        for test in suite.tests:
            for tag in test.tags:
                if tag.startswith("REQ-"):
                    logging.debug(f"Found REQ tag: {tag} in test {test.name} in suite {suite_filename}")
                    req_map.setdefault(tag, []).append((suite_filename, test.name, test.status))

    logging.debug(f"REQ Map: {json.dumps(req_map, indent=2)}")

    reqs = db.get_requirements()
    logging.debug(f"Requirements List (REQs): {json.dumps(reqs, indent=2)}")

    for req_id in reqs:
        reqs[req_id]["linked_tests"] = [test[1] for test in req_map.get(req_id, [])]
        reqs[req_id]["test_results"] = req_map.get(req_id, [])

    logging.debug(f"Updated Requirements with Test Results: {json.dumps(reqs, indent=2)}")

    stats = _default_stat()
    detailed_stats = {
            "domains": defaultdict(_default_stat),
            "types": defaultdict(_default_stat)
    }

    # Process reqs
    for req in reqs.values():
        rdomain = req["domain"]
        rtype = req["type"]
        tested = bool(req["linked_tests"])
        total_tests = len(req["test_results"])
        passed_count = sum(1 for test in req["test_results"] if test[2] == "PASS")

        # Update global stats
        stats["total_requirements"] += 1
        if tested:
            stats["tested_requirements"] += 1
        stats["total_tests"] += total_tests
        stats["passed_tests"] += passed_count

        # Update domain stats
        d = detailed_stats["domains"][rdomain]
        d["total_requirements"] += 1
        if tested:
            d["tested_requirements"] += 1
        d["total_tests"] += total_tests
        d["passed_tests"] += passed_count

        # Update type stats
        t = detailed_stats["types"][rtype]
        t["total_requirements"] += 1
        if tested:
            t["tested_requirements"] += 1
        t["total_tests"] += total_tests
        t["passed_tests"] += passed_count

    _finalize(stats)
    for group in ["domains", "types"]:
        for key, val in detailed_stats[group].items():
            _finalize(val)

    # Output
    logging.debug(f"Final Stats: {json.dumps(stats, indent=2)}")
    logging.debug(f"Detailed Stats: {json.dumps(detailed_stats, indent=2)}")

    report = {
        "timestamp": datetime.now().isoformat(),
        "coverage": stats,
        "report": reqs,
        "detailed_report": {
            "domains": dict(detailed_stats["domains"]),
            "types": dict(detailed_stats["types"])
        }
    }
    writer = ReportWriter(report)

    for req_id, req_data in reqs.items():
        tests = req_data["linked_tests"]
        uuid = req_data.get("uuid")
        db.link_tests(tests=tests, uuid=uuid)

    writer.write(fmt, output, domain, req_type, detail)



# Copyright (c) 2025 Daniel Paredes (daleonpz)
# SPDX-License-Identifier: Apache-2.0

from raiz.utils.report_generator import ReportWriter

import sqlite3
from typing import Optional, List, Dict
import uuid
from pathlib import Path
from collections import defaultdict
import typer
from rich.console import Console
from rich.traceback import install
install(show_locals=True)

REQ_CACHE_DIR = Path(".req_cache")
REQ_CACHE_DIR.mkdir(exist_ok=True)
DB_PATH = REQ_CACHE_DIR / "requirements.db"

REQS_DIR = Path("requirements")
REQS_DIR.mkdir(exist_ok=True)
REQ_FILE = REQS_DIR / "requirements.yaml"

HEADER_STYLE = "bold magenta"
MAIN_COLUMN_STYLE = "cyan"

console = Console()

class Database:
    def __init__(self, db_path: str = str(DB_PATH)):
        self.db_path = db_path
        self.conn = None
        self.initialize()

    def initialize(self):
        """Initialize the database and create the requirements table if it doesn't exist."""
        with self as conn:
            conn.execute("""
                CREATE TABLE IF NOT EXISTS requirements (
                    uuid TEXT PRIMARY KEY,
                    seq_no INTEGER NOT NULL,
                    description TEXT NOT NULL,
                    type TEXT NOT NULL,
                    domain TEXT NOT NULL,
                    linked_tests TEXT DEFAULT ''
                )
            """)
            conn.commit()

    def __enter__(self):
        self.conn = sqlite3.connect(self.db_path)
        self.conn.row_factory = sqlite3.Row
        return self.conn

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.conn:
            self.conn.commit()
            self.conn.close()
            self.conn = None

class RequirementsDB:
    def __init__(self, database: Optional[Database] = None):
        self.database = database or Database()

    def add_requirement(self,  description: str, req_type: str, domain: str):
        with self.database as conn:
            cur = conn.execute("SELECT MAX(seq_no) FROM requirements")
            max_seq = cur.fetchone()[0] or 0
            seq_no = max_seq + 1
            conn.execute(
                "INSERT INTO requirements (uuid, seq_no, description, type, domain) VALUES (?, ?, ?, ?, ?)",
                (str(uuid.uuid4()), seq_no, description, req_type, domain)
            )
            conn.commit()

        console.print(f"[bold green]REQ-{seq_no:03} added[/bold green]")

    def rm(self, seq_no: int):
        """Remove a requirement by sequence number and renumber the rest."""
        with self.database as conn:
            conn.execute("DELETE FROM requirements WHERE seq_no = ?", (seq_no,))
            conn.commit()
            self.renumber_all()

        console.print(f"[bold red]REQ-{seq_no:03} removed[/bold red]")

    def renumber_all(self):
        with self.database as conn:
            cur = conn.execute("SELECT uuid FROM requirements ORDER BY seq_no ASC")
            uuids = [row["uuid"] for row in cur.fetchall()]
            for i, uid in enumerate(uuids, start=1):
                conn.execute("UPDATE requirements SET seq_no = ? WHERE uuid = ?", (i, uid))
            conn.commit()


    def update_requirement(self, seq_no: int, field: str, value: str):
        VALID_FIELDS = {"description", "type", "domain"}

        with self.database as conn:
            cur = conn.execute("SELECT * FROM requirements WHERE seq_no = ?", (seq_no,))
            row = cur.fetchone()
            if not row:
                console.print("[bold red]Requirement REQ-{seq_no:03} not found.[/bold red]")
                raise typer.Exit()

            if field and value:
                if field not in VALID_FIELDS:
                    typer.echo(f"Invalid field: {field}. Must be one of: {', '.join(VALID_FIELDS)}.")
                    raise typer.Exit()
                conn.execute(f"UPDATE requirements SET {field} = ? WHERE seq_no = ?", (value, seq_no))
                conn.commit()
                console.print(f"[bold green]Updated REQ-{seq_no:03}: set {field} to '{value}'[/bold green]")
                return

            console.print(f"[bold yellow]Updating REQ-{seq_no:03} interactively. Leave blank to keep existing values.[/bold yellow]")

            for f in VALID_FIELDS:
                current = row[f]
                new_val = typer.prompt(f"{f.capitalize()} [{current}]", default=current)
                conn.execute(f"UPDATE requirements SET {f} = ? WHERE seq_no = ?", (new_val, seq_no))

            conn.commit()
            console.print(f"[bold green]REQ-{seq_no:03} updated successfully![/bold green]")

    def show_requirements(self, req_type: Optional[str] = None, domain: Optional[str] = None, json: Optional[bool] = False):
        with self.database as conn:
            query = "SELECT * FROM requirements WHERE 1=1"
            filters = []
            args = []

            if req_type:
                filters.append("type = ?")
                args.append(req_type)

            if domain:
                filters.append("domain = ?")
                args.append(domain)

            if filters:
                query += " AND " + " ".join(filters)

            cur = conn.execute(query, args)
            rows = cur.fetchall()

            if not json:
                req_table = [["REQ-ID", "Description", "Type", "Domain"]]
                for row in rows:
                    req_table.append([
                        f"REQ-{row['seq_no']:03}",
                        row["description"],
                        row["type"],
                        row["domain"]
                    ])
                if not req_table:
                    console.print("[bold yellow]No requirements found.[/bold yellow]")
                    return

                ReportWriter.print_table_console(req_table, title=None, with_title= False)
            else:
                req_list = []
                for row in rows:
                    req_list.append({
                        "id": f"REQ-{row['seq_no']:03}",
                        "uuid": row["uuid"],
                        "description": row["description"],
                        "type": row["type"],
                        "domain": row["domain"],
                        "linked_tests": row["linked_tests"].split(",") if row["linked_tests"] else [],
                    })

                ReportWriter.print_table_json(req_list)


    def show_characteristic(self, char: str):
        """Show all requirements with a specific characteristic."""

        if char not in ["type", "domain"]:
            console.print(f"[bold red]Invalid characteristic: {char}[/bold red]")
            return

        table = [[char.capitalize()]]
        with self.database:
            values = self.get_characteristic(char)
            for value in values:
                table.append([value])

            ReportWriter.print_table_console(table, title=None, with_title = False)

    def get_characteristic(self, char: str) -> List[str]:
        """Get all unique values for a specific characteristic."""
        if char not in ["type", "domain"]:
            console.print(f"[bold red]Invalid characteristic: {char}[/bold red]")
            return []

        with self.database as conn:
            cur = conn.execute(f"SELECT DISTINCT {char} FROM requirements")
            values = [row[char] for row in cur.fetchall()]
        return sorted(values)

    def get_stats_by_characteristic(self, char: str, value: str, report = None
                                    ) -> Dict[str, int]:
        """Get the number of requirements for each value of a characteristic."""
        if char not in ["type", "domain"]:
            console.print(f"[bold red]Invalid characteristic: {char}[/bold red]")
            return {}

        with self.database as conn:
            cur = conn.execute(f"SELECT COUNT(*) FROM requirements WHERE {char} = ?", (value,))
            total = cur.fetchone()[0]

            cur = conn.execute(
                f"SELECT uuid FROM requirements WHERE {char} = ? AND linked_tests != ''",
                (value,)
            )

            fetched_data = cur.fetchall()
            total_tested = len(fetched_data)

            total_passed = [ report[row["uuid"]]["STATUS"] for row in fetched_data if row["uuid"] in report ] \
                .count("PASSED") if report else 0

        return {"total": total, "tested": total_tested, "passed": total_passed}

    def get_requirements(self) -> Dict[str, Dict[str, str]]:
        """Get all requirements as a list of dictionaries."""
        with self.database as conn:
            cur = conn.execute("SELECT * FROM requirements ORDER BY seq_no ASC")
            reqs = defaultdict(dict)
            for row in cur.fetchall():
                reqs[f"REQ-{row['seq_no']:03}"] = {
                    "uuid": row["uuid"],
                    "description": row["description"],
                    "type": row["type"],
                    "domain": row["domain"],
                    "linked_tests": row["linked_tests"].split(",") if row["linked_tests"] else [],
                }
        return reqs

    def get_requirements_uuids(self) -> Dict[str, str]:
        """Get all requirements as a mapping of REQ-ID to UUID."""
        with self.database as conn:
            cur = conn.execute("SELECT uuid, seq_no FROM requirements")
            req_dict = {f"REQ-{row['seq_no']:03}": row["uuid"] for row in cur.fetchall()}

        return req_dict

    def link_tests(self, tests: list = None, uuid: str = None):
        """Link tests to a requirement by UUID."""

        # tests can be deleted or empty, so we only check if uuid is provided
        if not uuid:
            console.print("[bold red]No UUID provided for linking.[/bold red]")
            return

        if not isinstance(tests, list):
            console.print("[bold red]Tests must be provided as a list.[/bold red]")
            return

        with self.database as conn:
            conn.execute(
                "UPDATE requirements SET linked_tests = ? WHERE uuid = ?",
                (",".join(tests), uuid),
            )


    def create_from_yaml(self, reqs: List[Dict[str, str]]):
        """Create requirements from a YAML-like structure."""
        if not reqs:
            console.print("[bold red]No requirements provided for creation.[/bold red]")
            return

        with self.database as conn:
            conn.execute("DELETE FROM requirements")
            for idx, req in enumerate(reqs, start=1):
                conn.execute(
                    "INSERT INTO requirements (uuid, seq_no, description, type, domain, linked_tests) VALUES (?, ?, ?, ?, ?, ?)",
                    (
                        req["uuid"],
                        idx,
                        req["description"],
                        req["type"],
                        req["domain"],
                        ",".join(req.get("linked_tests", []))
                    )
                )
            conn.commit()





# Copyright (c) 2025 Daniel Paredes (daleonpz)
# SPDX-License-Identifier: Apache-2.0

import sqlite3
from pathlib import Path

REQ_CACHE_DIR = Path(".req_cache")
REQ_CACHE_DIR.mkdir(exist_ok=True)
DB_FILE = REQ_CACHE_DIR / "requirements.db"

def db_conn():
    conn = sqlite3.connect(DB_FILE)
    conn.row_factory = sqlite3.Row
    return conn

def init_db():
    with db_conn() as conn:
        conn.execute("""
            CREATE TABLE IF NOT EXISTS requirements (
                uuid TEXT PRIMARY KEY,
                seq_no INTEGER UNIQUE,
                description TEXT NOT NULL,
                type TEXT NOT NULL,
                domain TEXT NOT NULL,
                linked_tests TEXT DEFAULT ''
        """)
        conn.commit()

def renumber_all():
    with db_conn() as conn:
        cur = conn.execute("SELECT uuid FROM requirements ORDER BY seq_no ASC")
        uuids = [row["uuid"] for row in cur.fetchall()]
        for i, uid in enumerate(uuids, start=1):
            conn.execute("UPDATE requirements SET seq_no = ? WHERE uuid = ?", (i, uid))
        conn.commit()




# Copyright (c) 2025 Daniel Paredes (daleonpz)
# SPDX-License-Identifier: Apache-2.0

import typer
import raiz.commands as commands
from importlib.metadata import version
from rich.traceback import install
install(show_locals=True)

app = typer.Typer(help="Raiz CLI - Manage your requirements from the command line")
show_app = typer.Typer()
sync_app = typer.Typer()

app.add_typer(show_app, name="show", help="Show requirements, types, or domains")
app.add_typer(sync_app, name="sync", help="Sync requirements with YAML file")

# Add subcommands
app.command("trace")(commands.trace)
app.command("add")(commands.add)
app.command("rm")(commands.remove)
app.command("update")(commands.update)

show_app.command("domains")(commands.show_domains)
show_app.command("types")(commands.show_types)
show_app.command("all")(commands.show_requirements)

sync_app.command("to-yaml")(commands.sync_to_yaml)
sync_app.command("from-yaml")(commands.sync_from_yaml)

def main():
    """
    Main entry point for the CLI application.
    """
    app()

if __name__ == "__main__":
    main()
